@@ -1,3 +1,3 @@
 from .main import Cerebrate, Options
 
-__all__ = ['Cerebrate']
+__all__ = ["Cerebrate", "Options"]
