@@ -1,6 +1,6 @@
 from typing import List
 import requests
-from .error import AuthenticationError, ArgumentValidationError, ServerError
+from .error import AuthenticationError, ArgumentValidationError, ServerError, OutOfLimitError
 
 UNKNOWN_ERROR = ServerError(
     message="Unknown error.", code=500, status="Internal server error"
@@ -16,7 +16,7 @@ class Options:
         frequency_penalty: float = 0,
         presence_penalty: float = 0,
         stop: [str] = ["Q:"],
-        best_of: int = 1,
+        best_of: int = 1
     ):
         self.temperature = temperature
         self.max_tokens = max_tokens
@@ -33,7 +33,7 @@ class Cerebrate:
         self._url = url
 
     def predict(
-        self, task: str, examples: [str], query: str, options: Options = Options()
+        self, task: str, examples: [str], query: str, options: Options = None
     ) -> List[str]:
         prompt = task
         if len(examples):
@@ -44,20 +44,32 @@ class Cerebrate:
         response = self.__send_request(prompt, options)
         return list(response)
 
-    def raw(self, prompt: str, options: Options = Options()) -> List[str]:
+    def raw(self, prompt: str, options: Options = None) -> List[str]:
         response = self.__send_request(prompt, options)
 
         return list(response)
 
-    # def match(self):
-    #     raise Exception("Not implemented!")
+    def preset_predict(self, preset_id: str, variables=None, options: Options = None):
+        if variables is None:
+            variables = {}
 
-    # def recommend(self):
-    #     raise Exception("Not implemented!")
+        response = self.__send_request("default", options, preset_id, variables)
 
-    def __send_request(self, prompt: str, options: Options):
-        body = {
-            "data": {
+        return list(response)
+
+    def __send_request(self, prompt: str, options: Options = None, preset_id: str = "", variables=None):
+        params = dict()
+        body = dict()
+
+        if preset_id:
+            body["presetId"] = preset_id
+            if options:
+                params['options'] = 'true'
+
+        if not options:
+            options = Options()
+
+        body["data"] = {
                 "prompt": prompt,
                 "temperature": options.temperature,
                 "maxTokens": options.max_tokens,
@@ -66,12 +78,14 @@ class Cerebrate:
                 "presencePenalty": options.presence_penalty,
                 "stop": options.stop,
                 "bestOf": options.best_of,
+                "source": "SDK_PYTHON"
             }
-        }
+        body["parameters"] = variables
 
         response = requests.post(
             self._url,
             json=body,
+            params=params,
             headers={
                 "Authorization": self._api_key,
                 "Content-Type": "application/json",
@@ -89,11 +103,17 @@ class Cerebrate:
             raise UNKNOWN_ERROR
 
         err_code = extensions.get("code")
+
         if err_code == "UNAUTHENTICATED":
             raise AuthenticationError(
                 message=resp_json.get("message"), code=401, status="Unauthorized"
             )
 
+        if err_code == "OUT_OF_LIMIT":
+            raise OutOfLimitError(
+                message=resp_json.get("message"), code=401, status="Out of limit"
+            )
+
         if err_code == "BAD_USER_INPUT":
             raise ArgumentValidationError(
                 message=resp_json.get("message"), code=400, status="Bad request"
